<?php

namespace PostLabelCenter\Components;

use Acl\GreenField\Exceptions\GreenFieldResponseException;
use Acl\GreenField\Exceptions\GreenFieldSDKException;
use Acl\GreenField\GreenField;
use Acl\GreenField\Modules\Plc\Shipment;
use Acl\GreenField\Modules\Plc\Shipment\Address;
use Acl\GreenField\Modules\Plc\Shipment\ColloRow;
use Acl\GreenField\Modules\Plc\Shipment\ColloRowArticle;
use Acl\GreenField\Modules\Plc\Shipment\DocumentEntry;
use Acl\GreenField\Modules\Plc\Shipment\Printer;
use GuzzleHttp\Client as HttpClient;
use PostLabelCenter\Models\PostLabelPluginConfiguration;
use PostLabelCenter\Models\PostLabelPluginConfigurationAddress;
use PostLabelCenter\Models\PostLabelPluginConfigurationBank;
use PostLabelCenter\Models\PostLabelPluginConfigurationContract;
use PostLabelCenter\Models\ShippingSet;
use Shopware\Components\HttpClient\GuzzleFactory;
use Shopware\Components\Logger;
use Shopware\Models\Order\Order;
use Symfony\Component\Validator\Constraints\DateTime;
use WebDriver\Exception;
use Zend_Cache_Core;

class GreenfieldService
{
    /**
     * Plugin logger
     *
     * @var \Shopware\Components\Logger
     */
    protected $logger;

    /**
     * Guzzle client factory
     *
     * @var \Shopware\Components\HttpClient\GuzzleFactory
     */
    protected $clientFactory;

    /**
     * Session handler
     *
     * @var \Zend_Cache_Core
     */
    protected $cache;

    /**
     * Greenfield Library instance
     *
     * @var \Acl\GreenField\GreenField
     */
    protected $gf;

    /**
     * Config instance
     *
     * @var array
     */
    protected $config;

    /**
     * Authenticated
     *
     * @var bool
     */
    protected $authenticated = false;

    /**
     * Auth token
     *
     * @var int
     */
    protected $authToken = 0;

    protected $imageMapping = [
//      'AN' => '',
        'AV' => '',
        'IV' => '',
        'IZ' => '',
        'ZU' => ''
    ];


    public function __construct(
        Logger          $logger,
        Zend_Cache_Core $cache,
        GuzzleFactory   $clientFactory
    )
    {
        $this->logger = $logger;
        $this->cache = $cache;
        $this->clientFactory = $clientFactory;
        if ($this->loadConfig(true)) {
            $this->initLib();
        }
    }

    public function getFirstAddress($shopId = null)
    {
        $addressRepositry = Shopware()->Models()->getRepository(PostLabelPluginConfigurationAddress::class);
        if ($shopId) {
            $allAddresses = $addressRepositry->findBy(["activeShopId" => $shopId]);
        }
        if (!$allAddresses) {
            $allAddresses = $addressRepositry->findAll();
        }
        if (!$allAddresses || empty($allAddresses) || !isset($allAddresses[0])) {
            return false;
        }
        return $allAddresses[0];
    }

    public function getFirstContract()
    {
        $contractRepositry = Shopware()->Models()->getRepository(PostLabelPluginConfigurationContract::class);
        $allContracts = $contractRepositry->findAll();
        if (!$allContracts || empty($allContracts) || !isset($allContracts[0])) {
            return false;
        }
        return $allContracts[0];
    }


    // TODO: skippingCheck Should not be needed ...
    protected function loadConfig($skipCheck = false)
    {
        $configuration = $this->getFullConfig();
        $firstContract = $this->getFirstContract();

        try {
            if (!$configuration) {
                $this->logger->error(
                    "Error ",
                    ["msg" => "Ungültige oder fehlende Österreichische Post Label Center (PLC) Pluginkonfiguration. Bitte prüfen Sie Ihre PLC-Einstellungen im Menü 'Einstellungen > PostPluginConfiguration'"]
                );
                return false;
            }
            $configurationArray = Shopware()->Models()->toArray($configuration);
            if (is_array($configurationArray) === false || ($config = reset($configurationArray)) === false) {
                // @todo: handle config error
                return false;
            }
        } catch (\Exception $e) {
            if (!$skipCheck) return false;
        }

        $this->setConfig($configuration, $firstContract);

        return true;
    }

    public function getFullConfig()
    {
        $repository = Shopware()->Models()->getRepository(PostLabelPluginConfiguration::class);
        $fullConfig = $repository->findAll();
        if (!$fullConfig || empty($fullConfig) || !isset($fullConfig[0])) {
            return false;
        }
        $fullConfig = $fullConfig[0];
        return $fullConfig;
    }

    protected function initLib()
    {
        $client = new HttpClient(
            array(
                "base_uri" => $this->config["apiUrl"]
            )
        );

        $this->gf = new GreenField(
            $client,
            $this->config["apiUrl"],
            "plc"
        );

        //Shopware::VERSION consant is removed since shopware 5.6
        if (defined('\Shopware::VERSION')) {
            $shopwareVersion = \Shopware::VERSION;
        } else {
            $shopwareVersion = Shopware()->Container()->getParameter('shopware.release.version');
        }

        $this->gf->setLogger($this->logger);
        $this->gf->setShopType("shopware"); // hardcoded, duh
        $this->gf->setShopVersion($shopwareVersion);
        $this->gf->setPluginVersion("1.3.4"); //TODO GET VERSION FROM services.xml
    }

    public function getConfig()
    {
        return $this->config;
    }


    public function setConfig(PostLabelPluginConfiguration $config, ?PostLabelPluginConfigurationContract $contract = null)
    {
        $this->config = [
            "apiUrl" => $config->getApiUrl(),
            "orgUnitID" => $contract ? (int)$contract->getContractnumber() : 0,
            "license" => $config->getLicense(),
            "orgUnitGuID" => $contract ? $contract->getUnitGUID() : '',
            "clientId" => $contract ? $contract->getLicense() : ''
        ];

        return $this;
    }

    public function importMultipleShipment($orders = null)
    {
        $orderRepository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        $shipments = [];
        foreach ($orders as $currentOrder) {

            $orderNumber = $currentOrder->ordernumber;
            if (!$orderNumber) {
                $orderNumber = $currentOrder->orderNumber;
            }
            $labelType = $currentOrder->labelType;
            $order = $orderRepository->findOneBy(["number" => $orderNumber]);
            if ($currentOrder) {
                $orderDataOverwrite = json_decode(json_encode($currentOrder), true);
            }
            if (!$order) {
                // todo: what?
                exit;
            }
            $billing = $order->getBilling();
            $customer = $billing->getCustomer();
            $shipping = $order->getShipping();
            $dispatch = $order->getDispatch();
            $dispatchId = $dispatch->getId();
            if (!$shipments[$dispatchId]) {
                // used shipping set
                try {
                    $repository = Shopware()->Models()->getRepository(ShippingSet::class);
                    $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

                    if (!$shippingSet) {
                        return new \Exception('Keine gespeicherte Konfiguration gefunden für Ihre Versandart ');
                    }

                    $features = $shippingSet->getFeatures();
                    $features = json_decode($features);

                    // compare initial user contactnumber (orgUnitGuID) with saved contractnumber in shipping set
                    // eventualy use the one from shippingset

                    $fullConfig = $this->getFullConfig();
                    $shopID = $order->getShop()->getId();
                    $defaultPluginAddress = $this->getFirstAddress($shopID);

                    if (!$fullConfig) {
                        return new \Exception('Keine Plugin Konfiguration gefunden!');
                    }
                    $overriddenConfig = $this->getOverriddenConfig($shippingSet);

                    if ($overriddenConfig) {
                        $shipments[$dispatchId]["meta"] = $overriddenConfig;
                    }
                } catch (\Exception $e) {
                    // eventual implement error handling
                    return $e;
                }

            }
            $shipment = new Shipment();
            foreach ($features as $feature) {

                if ($feature->checked) {
                    $tmpFeature = array(
                        'name' => $feature->name,
                        'thirdPartyID' => $feature->thirdPartyID
                    );
                    // nachnahme
                    if ($feature->thirdPartyID == '006' || $feature->thirdPartyID == '022') {
                        //63 - Höherversicherung
                        $senderBank = $orderDataOverwrite["senderBank"];
                        $invoiceAmount = $orderDataOverwrite["orderData"]["invoiceAmount"];
                        $tmpFeature = array_merge($tmpFeature, $this->getAdditionalImportInfo($order, $senderBank, $invoiceAmount));
                    } else if ($feature->thirdPartyID == '063') {
                        $higherInsurance = $this->getHigherInsurance($order->getId());

                        $higherInsuranceFeature = ["value1" => $higherInsurance, "value2" => "EUR"];
                        $tmpFeature = array_merge($tmpFeature, $higherInsuranceFeature);

                    } else if ($feature->thirdpartyID == '142') {
                        $shipment->setCustomDataBit(true);
                    }

                    $shipment->addFeature($tmpFeature);
                }
            }

            $orderAttributes = $this->getOrderAttributes($order->getId());
            $isBranchDelivery = false;

            if ($branch = $orderAttributes['branch']) {
                $isBranchDelivery = true;
                try {
                    $branchObject = json_decode($branch);
                    if (!$branchObject->additionalBranchData->branchKey || !$branchObject->additionalBranchData->type) {
                        // todo return exception
                        throw new \Exception('No branch data in shop order');
                    }
                    $shipment->setBranchKey($branchObject->additionalBranchData->branchKey);
                    $shipment->setBranchKeyType($branchObject->additionalBranchData->type);

                } catch (\Exception $e) {
                    // todo implement exception handling
                    die($e->getMessage());
                }
            }

            $bill = new Address();
            $bill->email = $customer->getEmail();
            $isShipping = false;
            if ($shipping->getStreet() && $shipping->getZipCode() && $shipping->getCity()) {
                $isShipping = true;
            }


            if ($receiverAddress = $orderDataOverwrite["receiverAddressValues"]) {
                $receiverAddress = json_decode($receiverAddress, true);
                $bill->firstname = $receiverAddress["firstName"];
                $bill->lastname = $receiverAddress["lastName"];
                $bill->company = $receiverAddress["company"];
                $bill->department = $receiverAddress["department"];
                $bill->address = $receiverAddress["street"];
                $bill->houseNumber = $receiverAddress["additionalAddressLine1"];
                $bill->name4 = $receiverAddress["additionalAddressLine2"];
                $bill->postalCode = $receiverAddress["zip"];
                $bill->city = $receiverAddress["city"];
                $bill->phone = $receiverAddress["phone"];
                $bill->countryIso2 = $receiverAddress["country"];
            } else {
                $firstName = $isShipping ? $shipping->getFirstName() : $billing->getFirstName();
                $lastName = $isShipping ? $shipping->getLastName() : $billing->getLastName();
                $company = $isShipping ? $shipping->getCompany() : $billing->getCompany();
                $department = $isShipping ? $shipping->getDepartment() : $billing->getDepartment();
                $bill->firstname = $isBranchDelivery ? $shipping->getDepartment() : $firstName;
                $bill->lastname = $isBranchDelivery ? $billing->getFirstName() . ' ' . $billing->getLastName() : $lastName;
                $bill->company = $isBranchDelivery ? '' : $company;
                $bill->department = $department;
                $address = $isShipping ? $shipping->getStreet() : $billing->getStreet();
                $postalCode = $isShipping ? $shipping->getZipCode() : $billing->getZipCode();
                $city = $isShipping ? $shipping->getCity() : $billing->getCity();

                $bill->address = $address;
                $bill->houseNumber = '';
                $bill->postalCode = $postalCode;
                $bill->city = $city;
                $bill->countryIso2 = $isShipping ? $shipping->getCountry()->getIso() : $billing->getCountry()->getIso();
                $bill->additionalAddr = $isBranchDelivery ? $shipping->getAdditionalAddressLine1() : '';
            }
            $shipper = new Address();
            $pluginConfig = $this->getFullConfig();
            if ($senderAddress = $orderDataOverwrite["senderAddressValues"]) {
                $senderAddress = json_decode($senderAddress, true);
                $shipper->firstname = $senderAddress["firstName"];
                $shipper->lastname = $senderAddress["lastName"];
                $shipper->address = $senderAddress["street"];
                $shipper->company = $senderAddress["company"];

                $shipper->houseNumber = '';
                $shipper->postalCode = $senderAddress["zip"];
                $shipper->city = $senderAddress["city"];
                $shipper->phone = $senderAddress["phone"];
                $shipper->countryIso2 = $senderAddress["country"];
                $shipper->additionalAddr = '';
                $shipper->EORINumber = $senderAddress["eoriNumber"];
            } else {
                $shipper->address = $defaultPluginAddress->getStreet();
                $shipper->firstname = $defaultPluginAddress->getFirstName();
                $shipper->lastname = $defaultPluginAddress->getLastName();
                $shipper->address = $defaultPluginAddress->getStreet();
                $shipper->houseNumber = '';
                $shipper->postalCode = $defaultPluginAddress->getZip();
                $shipper->city = $defaultPluginAddress->getCity();
                $shipper->phone = $defaultPluginAddress->getPhone();
                $shipper->countryIso2 = $defaultPluginAddress->getCountry();
                $shipper->company = $defaultPluginAddress->getCompany();
                $shipper->additionalAddr = '';
                $shipper->EORINumber = $defaultPluginAddress->getEoriNumber();
            }
            $returnAddr = new Address();

            if ($returnAddress = $orderDataOverwrite["returnAddressValues"]) {

                $returnAddr->firstname = $returnAddress["firstName"];
                $returnAddr->lastname = $returnAddress["lastName"];
                $returnAddr->address = $returnAddress["street"];
                $returnAddr->postalCode = $returnAddress["zip"];
                $returnAddr->city = $returnAddress["city"];
                $returnAddr->phone = $returnAddress["phone"];
                $returnAddr->countryIso2 = $returnAddress["country"];
                $returnAddr->additionalAddr = '';
            } else {
                $infoName = $pluginConfig->getInfoName();
                $infoNameExtended = $pluginConfig->getInfoNameExtended();
                $infoPhone = $pluginConfig->getInfoPhone();
                $infoStreet = $pluginConfig->getInfoStreet();
                $infoZip = $pluginConfig->getInfoZip();
                $infoCity = $pluginConfig->getInfoCity();
                //$infoCountry = $pluginConfig->getInfoCountry();
                $infoCountry = 'AT';

                $returnAddr->email = Shopware()->Config()->get('mail') ?: '';//$customer->getEmail();
                $returnAddr->company = $infoName;
                $returnAddr->department = $infoNameExtended;
//      $shipper->lastname = $infoName;
                $returnAddr->address = $infoStreet;
                //$shipper->houseNumber = '';
                $returnAddr->postalCode = $infoZip;
                $returnAddr->city = $infoCity;
                $returnAddr->countryIso2 = $infoCountry;
            }
            $colloArticleList = [];
            $shipmentDocumentEntryList = [];
            if ($articleDatas = $orderDataOverwrite["articleDataValues"]) {
                if (is_string($articleDatas)) {
                    $articleDatas = json_decode($articleDatas, true);
                }
                foreach ($articleDatas as $articleData) {
                    $articleRow = new ColloRowArticle();
                    $articleRow->articleName = $articleData["name"];
                    $articleRow->articleNumber = $articleData["articleNumber"];
                    $articleRow->consumerUnitNetWeight = $articleData["weight"];
                    $articleRow->countryOfOriginID = $articleData["originCountry"];
                    $articleRow->hSTariffNumber = $articleData["hSTariffNumber"];
                    $articleRow->quantity = $articleData["quantity"];
                    $articleRow->unitID = $articleData["units"];
                    $articleRow->valueOfGoodsPerUnit = $articleData["price"];
                    $articleRow->customsOptionID = $articleData["customsOption"];
                    //TODO LOAD FROM BACKEND IDS
                    $articleRow->currencyID = "EUR";
                    $colloArticleList[$articleData["packageNumber"]][] = $articleRow;
                    if (key_exists("weight", $colloArticleList[$articleData["packageNumber"]])) {
                        $colloArticleList[$articleData["packageNumber"]]["weight"] += $articleData["weight"];
                    }
                }
            }
            //multiple
            $shipmentDocumentEntryList = [];
            if ($documentDatas = $orderDataOverwrite["documentDocumentDataValues"]) {
                if (is_string($documentDatas)) {
                    $documentDatas = json_decode($documentDatas, true);
                }
                foreach ($documentDatas as $documentData) {

                    //needs to be set on articlelist
                    $customsOptions = explode(",", $documentData["customsOption"]);
                    $documentNumbers = explode(",", $documentData["documentNumber"]);
                    $documentTypes = explode(",", $documentData["documentType"]);
                    foreach ($documentTypes as $key => $type) {
                        if (!$documentNumbers[$key]) {
                            continue;
                        }
                        $shipmentDocumentEntry = new DocumentEntry();
                        $shipmentDocumentEntry->number = $documentNumbers[$key];
                        $shipmentDocumentEntry->documentID = $type;
                        $shipmentDocumentEntry->quantity = 1;
                        $shipmentDocumentEntryList[] = $shipmentDocumentEntry;
                    }
                    $weight = $documentData["weight"];
                    //intern mapping
                    $packageNumber = $documentData["packageNumber"];
                }

            }
            if ($customsData = $orderDataOverwrite["customsData"]) {
                $customsData = json_decode($customsData, true);
                $shipment->setReturnOptionId($customsData["returnOption"]);
                $shipment->setReturnModeId($customsData["returnWay"]);
                $customsDescription = $customsData["customsDescription"];
                $shipment->setCustomsDescription($customsDescription);
            }


            // fill shipment info
            $shipment->setDeliveryId(strval($shippingSet->getProductID()));
            $shipment->setBillAddr($bill);
            $shipment->setShipperAddr($shipper);
            $shipment->setOrderId($order->getNumber());
            $shipment->setShipmentDocumentEntryList($shipmentDocumentEntryList);
            $colloRows = [];

            foreach ($colloArticleList as $packageNumber => $colloArticles) {
                $colloRow = new ColloRow();
                $colloRow->colloArticleRows = $colloArticles;
                $colloRows[] = $colloRow;
                foreach ($documentDatas as $documentData) {
                    if ($documentData["packageNumber"] === $packageNumber) {
                        $weight = $documentData["weight"];
                    }
                }
                $colloRow->height = 0;
                $colloRow->width = 0;
                $colloRow->length = 0;
                $colloRow->weight = $weight;

            }
            $shipment->setColloRow($colloRows);

            $shipment->setShipperReference($this->getShipperReference($order->getId()));
            //46 = ems international
            if ((string)($shippingSet->getProductID() == 46)) {
                if (!$customsDescription && $customsDescription !== "") {
                    $customsDescription = $this->getCustomsDescription($order->getId());
                }

                $emsErrors = false;
                if ($customsDescription) {
                    $shipment->setCustomsDescription($customsDescription);
                }
            }


            // add sum of product weights to shipment
            $repository = Shopware()->Models()->getRepository(\Shopware\Models\Article\Detail::class);
            $products = [];

            $counter = 0;
            $firstProduct = null;

            foreach ($order->getDetails() as $detail) {

                if ($detail->getMode() !== 0) {
                    continue;
                }
                $variant = $repository->findOneBy(['number' => $detail->getArticleNumber()]);

                $products[$detail->getArticleNumber()]["variant"] = $variant;
                $products[$detail->getArticleNumber()]["quantity"] = $detail->getQuantity();
                if ($counter === 0) {
                    $firstProduct = $variant->getArticle();
                }
                $counter += 1;
            }

            $weight = 0;
            foreach ($products as $product) {

                //no variant for this product
                if (!isset($product["variant"])) {
                    continue;
                }
                // find parent price
                if ($product["variant"]->getWeight() <= 0) {
                    $mainDetail = $product["variant"]->getArticle()->getMainDetail();
                    $weight += $mainDetail->getWeight() * $product["quantity"];
                    continue;
                }
                $weight += $product["variant"]->getWeight() * $product["quantity"];
            }

            if ($weight == 0) {
                $weight = 0.1;
            }


            //add phone numbers  and article description to features
            if ($firstProduct) {
                $articleDescription = (object)["name" => "articleDescription", "value1" => strip_tags($firstProduct->getName())];
                $shipment->addFeature($articleDescription);
            }
            //$shipperPhone = (object)["name" => "shipperPhone", "value1" => $infoPhone];
            //$recipentPhone = (object)["name" => "recipientPhone", "value1" => $billing->getPhone()];

            //$shipment->addFeature($shipperPhone);
            //$shipment->addFeature($recipentPhone);
            $shipment->addWeight((float)$weight);

            // id for webservice is actual our ordernumber
            $shopID = $order->getShop()->getId();
            $shipment->setShopId($shopID);

            // just for tracking purpose
            $shipment->setCustomerProduct($this->getCustomerProduct());


            $shipment->setGenerateLabel(true);

            if ($this->isDataImportOnly()) {
                $shipment->setGenerateLabel(false);
            } else {
                $printer = new Printer();
                $labelFormat = $pluginConfig->getPaperLayout();
                if ($labelFormat) {
                    $printer->layout = $labelFormat;
                }
                $shipment->setPrinter($printer);
            }
            if ($labelType !== 'BOTH') {
                $shipment->setDocType($labelType);
                $shipments[$dispatchId]["data"][] = $shipment;
            } else {
                // send requests
                $shipment->setDocType(Shipment::DOCTYPE_RETURN_LABEL);
                $shipments[$dispatchId]["data"][] = $shipment;
                $secondShipment = clone($shipment);
                $secondShipment->setDocType(Shipment::DOCTYPE_SHIPPING_LABEL);
                $shipments[$dispatchId]["data"][] = $secondShipment;
            }
        }
        return $this->__call("generateMultipleShippingPdfs", [$shipments]);


    }

    /*
     *         $orderDataForOverwrite = [
            'returnAddress' => $returnAddressValues,
            'senderAddress' => $senderAddressValues,
            'senderBank' => $senderBankValues,
            'receiverAddress' => $receiverAddressValues,
            'packageData' => $packageDataValues,
            'articleData' => $articleDataValues,
            'customsDuty' => $customsDutyValues,
            'documentDocumentData' => $documentDocumentDataValues,
            'documentArticleData' => $documentArticleDataValues,
            ];
     */
    public function importShipment($orderID = null, $labelType = Shipment::DOCTYPE_SHIPPING_LABEL, array $orderDataOverwrite = [])
    {
        $order = Shopware()->Models()->find('Shopware\Models\Order\Order', $orderID);
        if (!$order) {
            // todo: what?
            exit;
        }

        $billing = $order->getBilling();
        $customer = $billing->getCustomer();
        $shipping = $order->getShipping();
        $dispatch = $order->getDispatch();
        $dispatchId = $dispatch->getId();

        // used shipping set
        try {
            $repository = Shopware()->Models()->getRepository(ShippingSet::class);
            $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

            if (!$shippingSet) {
                return new \Exception('Keine gespeicherte Konfiguration gefunden für Ihre Versandart ');
            }

            $features = $shippingSet->getFeatures();
            $features = json_decode($features);

            // compare initial user contactnumber (orgUnitGuID) with saved contractnumber in shipping set
            // eventualy use the one from shippingset

            $fullConfig = $this->getFullConfig();

            if (!$fullConfig) {
                return new \Exception('Keine Plugin Konfiguration gefunden!');
            }
            $overriddenConfig = $this->getOverriddenConfig($shippingSet);

            if ($overriddenConfig) {
                $this->overrideAuth($overriddenConfig);

            }
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }

        $shipment = new Shipment();
        foreach ($features as $feature) {

            if ($feature->checked) {
                $tmpFeature = array(
                    'name' => $feature->name,
                    'thirdPartyID' => $feature->thirdPartyID
                );
                // nachnahme
                if ($feature->thirdPartyID == '006' || $feature->thirdPartyID == '022') {
                    //63 - Höherversicherung
                    $senderBank = $orderDataOverwrite["senderBank"];
                    $invoiceAmount = $orderDataOverwrite["orderData"]["invoiceAmount"];
                    $tmpFeature = array_merge($tmpFeature, $this->getAdditionalImportInfo($order, $senderBank, $invoiceAmount));

                } else if ($feature->thirdPartyID == '063') {
                    $higherInsurance = $this->getHigherInsurance($order->getId());

                    $higherInsuranceFeature = ["value1" => $higherInsurance, "value2" => "EUR"];
                    $tmpFeature = array_merge($tmpFeature, $higherInsuranceFeature);

                } else if ($feature->thirdPartyID == '142') {
                    $shipment->setCustomDataBit(true);
                }
                $shipment->addFeature($tmpFeature);

            }
        }


        $orderAttributes = $this->getOrderAttributes($order->getId());
        $isBranchDelivery = false;

        if ($branch = $orderAttributes['branch']) {
            $isBranchDelivery = true;
            try {
                $branchObject = json_decode($branch);
                if (!$branchObject->additionalBranchData->branchKey || !$branchObject->additionalBranchData->type) {
                    // todo return exception
                    throw new \Exception('No branch data in shop order');
                }
                $shipment->setBranchKey($branchObject->additionalBranchData->branchKey);
                $shipment->setBranchKeyType($branchObject->additionalBranchData->type);

            } catch (\Exception $e) {
                // todo implement exception handling
                die($e->getMessage());
            }
        }

        $bill = new Address();
        $bill->email = $customer->getEmail();
        $isShipping = false;
        if ($shipping->getStreet() && $shipping->getZipCode() && $shipping->getCity()) {
            $isShipping = true;
        }


        if ($receiverAddress = $orderDataOverwrite["receiverAddress"]) {
            $bill->firstname = $receiverAddress["firstName"];
            $bill->lastname = $receiverAddress["lastName"];
            $bill->company = $receiverAddress["company"];
            $bill->department = $receiverAddress["department"];
            $bill->address = $receiverAddress["street"];
            $bill->houseNumber = $receiverAddress["additionalAddressLine1"];
            $bill->name4 = $receiverAddress["additionalAddressLine2"];
            $bill->postalCode = $receiverAddress["zip"];
            $bill->city = $receiverAddress["city"];
            $bill->phone = $receiverAddress["phone"];
            //TODO COUNTRY
            $bill->countryIso2 = $receiverAddress["country"];
        } else {
            $firstName = $isShipping ? $shipping->getFirstName() : $billing->getFirstName();
            $lastName = $isShipping ? $shipping->getLastName() : $billing->getLastName();
            $company = $isShipping ? $shipping->getCompany() : $billing->getCompany();
            $department = $isShipping ? $shipping->getDepartment() : $billing->getDepartment();
            $bill->firstname = $isBranchDelivery ? $shipping->getDepartment() : $firstName;
            $bill->lastname = $isBranchDelivery ? $billing->getFirstName() . ' ' . $billing->getLastName() : $lastName;
            $bill->company = $isBranchDelivery ? '' : $company;
            $bill->department = $department;
            $address = $isShipping ? $shipping->getStreet() : $billing->getStreet();
            $postalCode = $isShipping ? $shipping->getZipCode() : $billing->getZipCode();
            $city = $isShipping ? $shipping->getCity() : $billing->getCity();

            $bill->phone = $billing->getPhone();
            $bill->address = $address;
            $bill->houseNumber = '';
            $bill->postalCode = $postalCode;
            $bill->city = $city;
            $bill->countryIso2 = $isShipping ? $shipping->getCountry()->getIso() : $billing->getCountry()->getIso();
            $bill->additionalAddr = $isBranchDelivery ? $shipping->getAdditionalAddressLine1() : '';
        }
        $shipper = new Address();
        $pluginConfig = $this->getFullConfig();
        $shopID = $order->getShop()->getId();
        $defaultPluginAddress = $this->getFirstAddress($shopID);

        if ($senderAddress = $orderDataOverwrite["senderAddress"]) {
            $shipper->firstname = $senderAddress["firstName"];
            $shipper->lastname = $senderAddress["lastName"];
            $shipper->address = $senderAddress["street"];
            $shipper->houseNumber = '';
            $shipper->postalCode = $senderAddress["zip"];
            $shipper->city = $senderAddress["city"];
            $shipper->phone = $senderAddress["phone"];
            $shipper->countryIso2 = $senderAddress["country"];
            $shipper->company = $senderAddress["company"];
            $shipper->additionalAddr = '';
            $shipper->EORINumber = $senderAddress["eoriNumber"];
        } else {
            $shipper->firstname = $defaultPluginAddress->getFirstName();
            $shipper->lastname = $defaultPluginAddress->getLastName();
            $shipper->address = $defaultPluginAddress->getStreet();
            $shipper->houseNumber = '';
            $shipper->postalCode = $defaultPluginAddress->getZip();
            $shipper->city = $defaultPluginAddress->getCity();
            $shipper->phone = $defaultPluginAddress->getPhone();
            $shipper->countryIso2 = $defaultPluginAddress->getCountry();
            $shipper->company = $defaultPluginAddress->getCompany();
            $shipper->additionalAddr = '';
            $shipper->EORINumber = $defaultPluginAddress->getEoriNumber();
        }
        $returnAddr = new Address();

        if ($returnAddress = $orderDataOverwrite["returnAddres"]) {

            $returnAddr->firstname = $returnAddress["firstName"];
            $returnAddr->lastname = $returnAddress["lastName"];
            $returnAddr->address = $returnAddress["street"];
            $returnAddr->postalCode = $returnAddress["zip"];
            $returnAddr->city = $returnAddress["city"];
            $returnAddr->phone = $returnAddress["phone"];
            $returnAddr->countryIso2 = $returnAddress["country"];
            $returnAddr->additionalAddr = '';
        } else {
            $infoName = $pluginConfig->getInfoName();
            $infoNameExtended = $pluginConfig->getInfoNameExtended();
            $infoPhone = $pluginConfig->getInfoPhone();
            $infoStreet = $pluginConfig->getInfoStreet();
            $infoZip = $pluginConfig->getInfoZip();
            $infoCity = $pluginConfig->getInfoCity();
            //$infoCountry = $pluginConfig->getInfoCountry();
            $infoCountry = 'AT';
            $returnAddr->phone = $infoPhone;
            $returnAddr->email = Shopware()->Config()->get('mail') ?: '';//$customer->getEmail();
            $returnAddr->company = $infoName;
            $returnAddr->department = $infoNameExtended;
            $returnAddr->address = $infoStreet;
            $returnAddr->postalCode = $infoZip;
            $returnAddr->city = $infoCity;
            $returnAddr->countryIso2 = $infoCountry;
        }
        $colloArticleList = [];
        $shipmentDocumentEntryList = [];

        if ($articleDatas = $orderDataOverwrite["articleData"]) {
            if (is_string($articleDatas)) {
                $articleDatas = json_decode($articleDatas, true);
            }

            foreach ($articleDatas as $articleData) {
                $articleRow = new ColloRowArticle();
                $articleRow->articleName = $articleData["name"];
                $articleRow->articleNumber = $articleData["articleNumber"];
                $articleRow->consumerUnitNetWeight = $articleData["weight"];
                $articleRow->countryOfOriginID = $articleData["originCountry"];
                $articleRow->hSTariffNumber = $articleData["hSTariffNumber"];
                $articleRow->quantity = $articleData["quantity"];
                $articleRow->unitID = $articleData["units"];
                $articleRow->valueOfGoodsPerUnit = $articleData["price"];
                $articleRow->customsOptionID = $articleData["customsOption"];
                //TODO LOAD FROM BACKEND IDS
                $articleRow->currencyID = "EUR";
                //$articleRow->showChildCollo = false;
                $colloArticleList[$articleData["packageNumber"]][] = $articleRow;
                if (array_key_exists("weight", $colloArticleList[$articleData["packageNumber"]])) {
                    $colloArticleList[$articleData["packageNumber"]]["weight"] += $articleData["weight"];
                }
            }
        }
        if ($documentDatas = $orderDataOverwrite["documentDocumentData"]) {
            if (is_string($documentDatas)) {
                $documentDatas = json_decode($documentDatas, true);
            }

            foreach ($documentDatas as $documentData) {

                //needs to be set on articlelist
                $customsOptions = explode(",", $documentData["customsOption"]);
                $documentNumbers = explode(",", $documentData["documentNumber"]);
                $documentTypes = explode(",", $documentData["documentType"]);

                foreach ($documentTypes as $key => $type) {
                    $shipmentDocumentEntry = new DocumentEntry();
                    $shipmentDocumentEntry->number = $documentNumbers[$key];
                    $shipmentDocumentEntry->documentID = $type;
                    $shipmentDocumentEntry->quantity = 1;

                    $shipmentDocumentEntryList[] = $shipmentDocumentEntry;
                }
                $weight = $documentData["weight"];
                //intern mapping
                $packageNumber = $documentData["packageNumber"];
            }
        }
        if ($customsData = $orderDataOverwrite["customsData"]) {
            $shipment->setReturnOptionId($customsData["returnOption"]);
            $shipment->setReturnModeId($customsData["returnWay"]);
            $customsDescription = $customsData["customsDescription"];
            $shipment->setCustomsDescription($customsDescription);
        }

        // fill shipment info
        $shipment->setDeliveryId(strval($shippingSet->getProductID()));
        $shipment->setBillAddr($bill);
        $shipment->setShipperAddr($shipper);
        $shipment->setOrderId($order->getNumber());
        $shipment->setShipmentDocumentEntryList($shipmentDocumentEntryList);
        $colloRows = [];

        foreach ($colloArticleList as $packageNumber => $colloArticles) {
            $colloRow = new ColloRow();
            $colloRow->colloArticleRows = $colloArticles;
            $colloRows[] = $colloRow;
            foreach ($documentDatas as $documentData) {
                if ($documentData["packageNumber"] === $packageNumber) {
                    $weight = $documentData["weight"];
                }
            }
            $colloRow->height = 0;
            $colloRow->width = 0;
            $colloRow->length = 0;
            $colloRow->weight = $weight;

        }
        $shipment->setColloRow($colloRows);
        $shipment->setShipperReference($this->getShipperReference($order->getId()));
        //46 = ems international
        if (strval($shippingSet->getProductID() == 46)) {
            if (!$customsDescription && $customsDescription !== "") {
                $customsDescription = $this->getCustomsDescription($order->getId());
            }
        }


        // add sum of product weights to shipment
        $repository = Shopware()->Models()->getRepository(\Shopware\Models\Article\Detail::class);
        $products = [];

        $counter = 0;
        $firstProduct = null;

        foreach ($order->getDetails() as $detail) {

            if ($detail->getMode() !== 0) {
                continue;
            }
            $variant = $repository->findOneBy(['number' => $detail->getArticleNumber()]);

            $products[$detail->getArticleNumber()]["variant"] = $variant;
            $products[$detail->getArticleNumber()]["quantity"] = $detail->getQuantity();
            if ($counter === 0) {
                $firstProduct = $variant->getArticle();
            }
            $counter += 1;
        }

        $weight = 0;
        foreach ($products as $product) {

            //no variant for this product
            if (!isset($product["variant"])) {
                continue;
            }
            // find parent price
            if ($product["variant"]->getWeight() <= 0) {
                $mainDetail = $product["variant"]->getArticle()->getMainDetail();
                $weight += $mainDetail->getWeight() * $product["quantity"];
                continue;
            }
            $weight += $product["variant"]->getWeight() * $product["quantity"];
        }

        if ($weight == 0) {
            $weight = 0.1;
        }


        //add phone numbers  and article description to features
        if ($firstProduct) {
            $articleDescription = (object)["name" => "articleDescription", "value1" => strip_tags($firstProduct->getName())];
            $shipment->addFeature($articleDescription);
        }
        //$shipperPhone = (object)["name" => "shipperPhone", "value1" => $infoPhone];
        //$recipentPhone = (object)["name" => "recipientPhone", "value1" => $billing->getPhone()];

        //$shipment->addFeature($shipperPhone);
        //$shipment->addFeature($recipentPhone);
        $shipment->addWeight((float)$weight);

        // id for webservice is actual our ordernumber
        $shopID = $order->getShop()->getId();
        $shipment->setShopId($shopID);

        // just for tracking purpose
        $shipment->setCustomerProduct($this->getCustomerProduct());


        $shipment->setGenerateLabel(true);

        if ($this->isDataImportOnly()) {
            $shipment->setGenerateLabel(false);
        } else {
            $printer = new Printer();
            $labelFormat = $pluginConfig->getPaperLayout();
            if ($labelFormat) {
                $printer->layout = $labelFormat;
            }
            $shipment->setPrinter($printer);
        }


        if ($labelType !== 'BOTH') {
            $shipment->setDocType($labelType);
            $response = $this->__call("generateShippingPdf", [$shipment]);
        } else {
            // send requests
            $shipment->setDocType(Shipment::DOCTYPE_RETURN_LABEL);
            $returnResponse = $this->__call("generateShippingPdf", [$shipment]);

            $shipment->setDocType(Shipment::DOCTYPE_SHIPPING_LABEL);

            $shippingResponse = $this->__call("generateShippingPdf", [$shipment]);

            $response = ["shippingResponse" => $shippingResponse, "returnResponse" => $returnResponse];
        }

        return $response;
    }

    //gets customsdescription attribute from the db instead of the service (service saves too late)
    protected function getCustomsDescription($orderId)
    {
        if ((int)$orderId) {
            $sql = "select customsdescription  from s_order_attributes
                	where orderID = $orderId";
            $query = Shopware()->Container()
                ->get('dbal_connection')
                ->executeQuery($sql);

            return $query->fetchColumn();
        }

        return false;
    }

    //gets higher_insurance attribute from the db instead of the service (service saves too late)
    protected function getHigherInsurance($orderId)
    {
        if ((int)$orderId) {
            $sql = "select higher_insurance  from s_order_attributes
                	where orderID = $orderId";
            $query = Shopware()->Container()
                ->get('dbal_connection')
                ->executeQuery($sql);

            return $query->fetchColumn();
        }

        return false;
    }

    //gets shipper_reference_2 attribute from the db instead of the service (service saves too late)
    protected function getShipperReference($orderId)
    {
        if ((int)$orderId) {
            $sql = "select shipper_reference_2  from s_order_attributes
                	where orderID = $orderId";
            $query = Shopware()->Container()
                ->get('dbal_connection')
                ->executeQuery($sql);

            return $query->fetchColumn();
        }

        return false;
    }

    public function getOrderAttributes($orderId)
    {
        $attributeLoader = Shopware()->Container()->get('shopware_attribute.data_loader');
        $orderAttributes = $attributeLoader->load('s_order_attributes', $orderId);

        return $orderAttributes;
    }

    public function getCustomerProduct()
    {
        //Shopware::VERSION consant is removed since shopware 5.6
        if (defined('\Shopware::VERSION')) {
            $shopwareVersion = \Shopware::VERSION;
        } else {
            $shopwareVersion = Shopware()->Container()->getParameter('shopware.release.version');
        }

        return "Shopware " . $shopwareVersion;
    }

    public function isDataImportOnly()
    {
        if (!$fullConfig = $this->getFullConfig()) {
            return false;
        }
        return $fullConfig->isDataImportOnly() == true;
    }

    public function __call($name, array $params)
    {
        if ($this->gf === null) {
            $this->initLib();
        }

        if ($this->authenticated === false) {
            $error = $this->auth();
            if ($error) {
                throw new \Exception($error);
            }
        }

        $response = call_user_func_array([$this->gf, $name], $params);
        return $response;
    }

    protected function auth()
    {
        $config = $this->config;

        unset($config["apiUrl"]);
        if (!$config) {
            $error = "Ungültige oder fehlende Österreichische Post Label Center (PLC) Pluginkonfiguration. Bitte prüfen Sie Ihre PLC-Einstellungen im Menü 'Einstellungen > PostPluginConfiguration'";
            $this->logger->error(
                "Error ",
                ["msg" => $error]
            );

            return $error;
        }
        $this->authenticated = $this->gf->authenticate($config);
    }

    public function cancelShipment($documentIds = null)
    {
        $response = $this->__call("cancelShipment", [$documentIds]);

        return $response;
    }

    public function getTrackingData($orderNumber = null, $trackingCode = null)
    {
        // todo - order check for codes realy necesary?
        if (!$trackingCode) {
            return array('noTrackingCode' => true);
        }

        try {
            $trackingArray = $this->getTrackingStatus($trackingCode);
        } catch (GreenFieldResponseException $e) {

            $this->logger->error(
                "Error fetching tracking status from the API",
                ["msg" => $e->getMessage()]
            );

            $response = json_decode($e->getResponse()->getBody());

            // special case - do trackign data for order
            if (!$response || strpos($response->error->extendedMessage,
                    'Es konnten keine Daten für Ihre Abfrage gefunden werden.') !== false) {
                return array('noTrackingData' => true);
            }
            return $response;

        } catch (GreenFieldSDKException $e) {
            $this->logger->error(
                "Error fetching tracking status from the API",
                ["msg" => $e->getMessage()]
            );

            return array('noTrackingData' => true);

        } catch (\Exception $e) {
            $this->logger->error(
                "Error fetching tracking status from the API",
                ["msg" => $e->getMessage()]
            );

            return array('noTrackingData' => true);
        }

        $path = 'custom/plugins/PostLabelCenter/Resources/views/frontend/_public/src/img/';
        $this->imageMapping = [
            'AV' => $path . 'posticon-paket-sendungsuebernahme.png',
            'IV' => $path . 'posticon-lkw.png',
            'IZ' => $path . 'posticon-paket-zustellung.png',
            'ZU' => $path . 'posticon-checkbox-checked.png',
        ];
        foreach ($trackingArray[0]->parcels as &$package) {
            foreach ($package->parcelEvents as &$packageEvent) {
                $dateTime = new \DateTime();
                $dateTime->setTimestamp($packageEvent->eventTimestamp / 1000);
                $date = $dateTime->format('d.m.Y H:i:s');
                $packageEvent->aclFormateDate = $date;
                // add icons
                if (isset($this->imageMapping[$packageEvent->trackingState])) {
                    $packageEvent->icon = $this->imageMapping[$packageEvent->trackingState];
                }
            }
        }
        return $trackingArray;
    }

    public function getTrackingStatus($trackingCode)
    {
        $response = $this->__call("getParcelDetail", [$trackingCode]);

        return $response->data;
    }

    public function checkFeatureParams($featureIDs)
    {
        $response = $this->__call("checkParams", array($featureIDs));

        return $response;
    }

    public function pluginConfigurationExists()
    {
        return (bool)$this->getPostPluginConfiguration();
    }

    public function getPostPluginConfiguration()
    {
        $repository = Shopware()->Models()->getRepository(PostLabelPluginConfiguration::class);
        $configurations = $repository->findAll();

        // todo - change later if multiple instances allowed
        return $configurations[0];
    }

    public function markOrderAsDownloaded($orderId, $documentId, $pdfDocument)
    {
        // save view into DB - usability
        $orderAttributes = $this->getOrderAttributes($orderId);
        if (!$labelsDownloadsList = $orderAttributes['post_labels_downloads']) {
            $labelsDownloadsList = '[]';
        }

        $labelsDownloadsList = json_decode($labelsDownloadsList);


        foreach ($labelsDownloadsList as &$labelDownload) {
            if ($labelDownload->documentid == $documentId) {
                $labelDownload->label_data->downloaded == true ? $documentDownloaded = true : $documentDownloaded = false;
                if (!$documentDownloaded) {
                    $labelDownload->label_data->downloaded = true;
                    $orderAttributes['post_labels_downloads'] = json_encode($labelsDownloadsList);
                    $attributePersister = Shopware()->Container()->get('shopware_attribute.data_persister');

                    try {
                        $attributePersister->persist($orderAttributes, 's_order_attributes', $orderId);
                        Shopware()->Models()->flush();
                    } catch (\Exception $e) {
                        $this->View()->assign([
                            'data' => $e->getMessage(),
                            'success' => false,
                        ]);

                        return;
                    }
                }
                break;
            }
        }
    }

    public function getOrderTrackingCodes($orderId)
    {
        $repository = Shopware()->Models()->getRepository(Order::class);
        if (!$order = $repository->findOneBy(['number' => $orderId])) {
            return false;
        }

        if (!$trackingCode = $order->getTrackingCode()) {
            return false;
        }

        return $trackingCode;
    }

    public function isOrderReturnAllowed()
    {
        if (!$fullConfig = $this->getFullConfig()) {
            return false;
        }

        return $fullConfig->isReturnOrderAllowed() == true;
    }

    public function getReturnReasons()
    {
        if (!$fullConfig = $this->getFullConfig()) {
            return false;
        }

        return $fullConfig->getReturnReasons();
    }

    public function getReturnTimeMax()
    {
        if (!$fullConfig = $this->getFullConfig()) {
            return false;
        }

        return $fullConfig->getReturnTimeMax();
    }

    public function getAdditionalImportInfo($order, $senderBank, $invoiceAmount)
    {
        if (!$senderBank || !is_array($senderBank)) {
            $bankRepository = Shopware()->Models()->getRepository(PostLabelPluginConfigurationBank::class);
            $bankEntries = $bankRepository->findAll();
            if (!$bankEntries) {
                return false;
            }
            $bank = $bankEntries[0];
            if (!$bank) {
                return false;
            }
            $invoiceAmount = (string)$order->getInvoiceAmount();
            $senderBank = ["iban" => $bank->getIban(), "bic" => $bank->getBic(), "bankAccountOwner" => $bank->getBankAccountOwner()];
        }

        return [
            'value1' => $invoiceAmount,
            'value2' => 'EUR',
            'value3' => $senderBank["iban"] . "|" . $senderBank["bic"] . "|" . $senderBank["bankAccountOwner"],
            'value4' => ''
        ];
    }

    public function getOverriddenConfig($shippingSet, $contractNumber = false)
    {
        if ($shippingSet) {
            $shippingContractNumber = $shippingSet->getContractNumber();
        }


        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationContract::class);
        $contracts = $repository->findAll();

        $contractData = Shopware()->Models()->toArray($contracts);
        if ($contractNumber) {
            foreach ($contractData as $contract) {
                if ($contract["contractnumber"] == (int)$contractNumber) {
                    return [
                        'orgUnitID' => (int)trim($contract["contractnumber"]),
                        'orgUnitGuID' => trim($contract["unitGUID"]),
                        'clientId' => trim($contract["license"])
                    ];
                }
            }
            return false;
        }
        if ($shippingContractNumber != '') {
            foreach ($contractData as $contract) {
                if ($contract["contractnumber"] == $shippingContractNumber) {
                    return [
                        'orgUnitID' => (int)trim($contract["contractnumber"]),
                        'orgUnitGuID' => trim($contract["unitGUID"]),
                        'clientId' => trim($contract["license"])
                    ];
                }
            }
        }

        return false;
    }

    public function getLogger()
    {
        return $this->logger;
    }

}
